% [u1,u2,x,t] = heatnonlinear(t_0,t_f,Nt,Nx,delta)
%
% Solves the heat equation that is considered in the paper
% Mazenc, Prieur, 2010
% this is a nonlinear parabolic equation
%
% with initial data u_0 = cos(x) 
% with periodic boundary conditions 
% using finite-differences in space and explicit time-stepping
% with a perturbation given by the external function perturb(t,delta)
% 
% inputs:
% t_0 is the initial time, 
% t_f is the final time, 
% Nx is the number of mesh-points, 
% Nt is the number of time steps
% delta is a parameter that is used for the perturbation
% PerturbCut is a parameter that is used for the perturbation
%
% outputs:
% u1 is the first component of the approximated solution
% u2 is the second component of the approximated solution
% x is the space vector (uniform between 0 and 2pi)
% t is the time vector (uniform between t0 and tf)

function [u1,u2,x,t] = heatnonlinear(t_0,t_f,Nx,Nt,delta,PerturbCut)

% keyboard
% define the mesh in space
dx = 2*pi/(Nx-1);
x = 0:dx:2*pi;
x = x';

% define the mesh in time
dt = (t_f-t_0)/Nt;
t = t_0:dt:t_f;

% define the ratio r
r2 = dt/dx^2;
r1 = dt/(2*dx);

% check the stability condition
if r2 > 0.5 
    disp('the numerical scheme is unstable')
    disp('choose a larger Nt or a smaller Nx')
    return
end

u1=inf*ones(Nx,Nt+1);
u2=inf*ones(Nx,Nt+1);

for i=1:Nx
  u1(i,1) = cos(x(i));
  %u2(i,1) = sin(x(i));
  u2(i,1) = 1;
end 

% for internal points, have
%    u_new(j) = u_old(j) + r*(u_old(j+1)-2*u_old(j)+u_old(j-1))
% for the two end-points, have
%    u_new(1) = u_old(1) + r*(u_old(2)-2*u_old(1)+u_old(Nx-1))     
%    u_new(Nx) = u_old(Nx) + r*(u_old(2)-2*u_old(Nx)+u_old(Nx-1))     
% clearly the endpoints are redundant: u(1)= u(Nx) at all times.  I just
% kept them around for plotting convenience.  

% when discretizing, the perturbation effect is multiplied by dt
dtdelta= dt*delta;

for j=1:Nt
  % left boundary condition
  % u1
  u1(1,j+1) = u1(1,j) + r2*(u1(2,j)-2*u1(1,j)+u1(Nx-1,j))...
              -r1*(u1(2,j)-u1(Nx-1,j))...
              +dt*u2(1,j)*(1+u1(1,j)^2)...
              +perturb(t(j),dtdelta,PerturbCut);
  % u2
  u2(1,j+1) = u2(1,j) + r2*(u2(2,j)-2*u2(1,j)+u2(Nx-1,j))...
              -dt*u1(1,j)*(1+u1(1,j)^2)...
              -dt*u2(1,j)*(2+u1(1,j)^2)...
              +perturb(t(j),dtdelta,PerturbCut);
  % in the middle
  for i=2:Nx-1
    % u1
    u1(i,j+1) = u1(i,j) + r2*(u1(i+1,j)-2*u1(i,j)+u1(i-1,j))...
                -r1*(u1(i+1,j)-u1(i-1,j))...
                +dt*u2(i,j)*(1+u1(i,j)^2)...
                +perturb(t(j),dtdelta,PerturbCut);
    % u2
    u2(i,j+1) = u2(i,j) + r2*(u2(i+1,j)-2*u2(i,j)+u2(i-1,j))...
                -dt*u1(i,j)*(1+u1(i,j)^2)...
                -dt*u2(i,j)*(2+u1(i,j)^2)...
                +perturb(t(j),dtdelta,PerturbCut);
  end
  % right boundary condition
  u1(Nx,j+1) = u1(Nx,j) + r2*(u1(2,j)-2*u1(Nx,j)+u1(Nx-1,j))...
               -r1*(u1(2,j)-u1(Nx-1,j))...
               +dt*u2(Nx,j)*(1+u1(Nx,j)^2)...
               +perturb(t(j),dtdelta,PerturbCut);
  u2(Nx,j+1) = u2(Nx,j) + r2*(u2(2,j)-2*u2(Nx,j)+u2(Nx-1,j))...
               -dt*u1(Nx,j)*(1+u1(Nx,j)^2)...
               -dt*u2(Nx,j)*(2+u1(Nx,j)^2)...
               +perturb(t(j),dtdelta,PerturbCut);
end

